
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""


### Imports ###

import socket
import select
import weakref
import sys
import time

from errno import EINTR, EBADF, EWOULDBLOCK


### Module Constants ###

# Global socket lists
_reads = []
_writes = []

# Join both lists
_modes = _reads, _writes

# Aliases
_select = select.select

# Constants
READ = 0
WRITE = 1


### Exceptions ###

class Error(Exception):
    pass

class SelectError(Error):
    pass

class ServerError(Error):
    pass

class SocketError(Error):
    """An error ocurred with a socket"""

class ConnectionError(SocketError):
    """An error ocurred with a connection"""

class ClientClosed(ConnectionError):
    """-1"""

class AddressAlreadyInUse(ConnectionError):
    """errno code: 98"""

class AddressNotAvailable(ConnectionError):
    """errno code: 99"""

class NetworkIsDown(ConnectionError):
    """errno code: 100"""

class NetworkUnreachable(ConnectionError):
    """errno code: 101"""

class NetworkReset(ConnectionError):
    """errno code: 102"""

class ConnectionAborted(ConnectionError):
    """errno code: 103"""

class ConnectionResetByPeer(ConnectionError):
    """errno code: 104"""

class NoBufferSpaceAvailable(ConnectionError):
    """errno code: 105"""

class SocketIsConnected(ConnectionError):
    """errno code: 106"""

class NotConnected(ConnectionError):
    """errno code: 107"""

class TCPTimeOut(ConnectionError):
    """errno code: 110"""

class ConnectionRefused(ConnectionError):
    """errno code: 111"""

class HostIsDown(ConnectionError):
    """errno code: 112"""

class HostUnreachable(ConnectionError):
    """errno code: 113"""

class ConnectionAlreadyInProgress(ConnectionError):
    """errno code: 114"""

_connErrorMap = {-1:ClientClosed,
                 98:AddressAlreadyInUse,
                 99:AddressNotAvailable,
                 100:NetworkIsDown,
                 101:NetworkUnreachable,
                 102:NetworkReset,
                 103:ConnectionAborted,
                 104:ConnectionResetByPeer,
                 105:NoBufferSpaceAvailable,
                 106:SocketIsConnected,
                 107:NotConnected,
                 110:TCPTimeOut,
                 111:ConnectionRefused,
                 112:HostIsDown,
                 113:HostUnreachable,
                 114:ConnectionAlreadyInProgress}

class MapSocketError(ConnectionError):
    """A especial exception that maps errno error codes to
    specific ConnectionError exceptions for each case.

    """
    # this could be easily done with __new__, but since exceptions are
    # still old style classes, this ugly hack is needed
    def __init__(self, reason):
        """
        @param reason: the (code, msg) tuple returned by a
            socket.error exception
            
        """
        code, msg = reason
        exc = _connErrorMap.get(code, ConnectionError)
        self.__class__ = exc
        ConnectionError.__init__(self)


### Sockets ###
        
class BaseSocket(object):

    family = socket.AF_INET
    type = socket.SOCK_STREAM

    def __init__(self, server, sock=None):
        self._server = weakref.ref(server)
        self.socket = sock or socket.socket(self.family, self.type)
        self.socket.setblocking(0)
        self.fileno = self.socket.fileno
        self.closed = False
        self.attrs = {}

    # Ugly, ugly, ugly hack... I don't want to create reference cycles
    # with the server, and it will outlive clients... of course, I
    # don't need property for this, but I don't want to need to
    # remember that this is a weakref
    server = property(lambda self: self._server())

    def __nonzero__(self):
        return not self.closed

    def close(self, reason=None):
        self._close()
    
    def _close(self, reason=None):
        try:
            try:
                self.socket.shutdown(2)
            except socket.error:
                self.socket.close()
        finally:
            self.closed = True


class ClientSocket(BaseSocket):

    bufsize = 8192

    def __init__(self, server, sock, addr):
        super(ClientSocket, self).__init__(server, sock)
        self.addr = addr
        self.attrs['host'], self.attrs['port'] = addr
        self.attrs['server'] = server.host
        self._buffer = ""

        self.connectionEstablished()

    def recv(self):
        try:
            data = self.socket.recv(self.bufsize)
        except socket.error, reason:
            if reason.args[0] == EWOULDBLOCK:
                return
            else:
                raise MapSocketError(reason.args)
        if not data:
            raise MapSocketError(('-1', 'Other side has closed connection'))

        self.dataReceived(data)

    # Callback to be called when the socket is ready for reading
    ready = recv

    def send(self, data):
        self._buffer += data

    def flush(self):
        try:
            sbytes = self.socket.send(self._buffer)
            sdata = self._buffer[:sbytes]
            self._buffer = self._buffer[sbytes:]
        except socket.error, reason:
            if reason.args[0] == EWOULDBLOCK:
                return ''
            else:
                raise MapSocketError(reason.args)
            
        return sdata
    
    def close(self, reason=None):
        self.server._close(self, reason=reason)

    def _close(self, reason=None):
        super(ClientSocket, self)._close(reason)
        self.connectionLost()

class ServerSocket(BaseSocket):

    def __init__(self, server, protocol):
        super(ServerSocket, self).__init__(server)
        self.addr = None
        self.name = name = protocol.__name__
        # self.protocol is a dinamically generated class, inheriting
        # both protocol and Clientsocket
        self.protocol = type(name, (protocol, ClientSocket), {})
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    def bind(self, host, port):
        try:
            self.socket.bind((host, port))
            self.addr = (host, port)
        except socket.error, reason:
            raise MapSocketError(reason.args)

    def listen(self, backlog=1):
        try:
            self.socket.listen(backlog)
        except socket.error, reason:
            raise MapSocketError(reason.args)

    def accept(self):
        try:
            sock, addr = self.socket.accept()
            return self.protocol(self.server, sock, addr)
        except socket.error, reason:
            raise MapSocketError(reason.args)

    # Callback to be called when the socket is ready for reading
    ready = accept


### Protocol ###

class BaseProtocol(object):
    """Base class for protocols

    """
    def connectionEstablished(self):
        """Callback called when the connection is established

        """

        raise NotImplementedError

    def connectionLost(self):
        """Callback called when the connection is lost

        """

        raise NotImplementedError


class RawProtocol(BaseProtocol):
    """Receives and send raw data

    """

    def dataReceived(self, data):
        """Callback called to process received data

        """

        raise NotImplementedError


class LineProtocol(RawProtocol):
    """Line oriented protocol. Send and receive data line by line


    """
    
    lineMode = True
    _rbuffer = ''
    delimiter = '\r\n'
    MAX = 16384

    def dataReceived(self, data):
        """Process received data into lines

        """
        
        self._rbuffer = self._rbuffer + data
        while self.lineMode:
            try:
                line, self._rbuffer = self._rbuffer.split(self.delimiter, 1)
            except ValueError:
                if len(self._rbuffer) > self.MAX:
                    line, self._rbuffer = self._rbuffer, ''
                    self.lineLengthExceeded(line)
                    return
                break
            else:
                linelength = len(line)
                if linelength > self.MAX:
                    line, self._rbuffer = self._rbuffer, ''
                    self.lineLengthExceeded(line)
                    return
                self.lineReceived(line)

        else:
            data, self._rbuffer = self._rbuffer, ''
            if data:
                self.rawDataReceived(data)

    def setLineMode(self, extra=''):
        """Enter line mode

        """
        
        self.lineMode = True
        self.dataReceived(extra)

    def setRawMode(self):
        """Enter Raw mode

        """
        
        self.lineMode = False

    def rawDataReceived(self, data):
        """Callback

        Should be overriden
        
        """
        
        raise NotImplementedError

    def lineReceived(self, line):
        """Callback

        Should be overriden
        
        """
        
        raise NotImplementedError

    def sendline(self, line):
        """Send a line. Use this method instead of 'send'

        """

        self.send(line + self.delimiter)

    def lineLengthExceeded(self, line):
        """Callback called when a line exceeeds max length

        This default implementation simply close the socket.

        Override it if this is not the desired behavior

        """

        self.close()


### Select ###

class Select(object):
    
    def _validate(self, socket):
        if not isinstance(socket, BaseSocket):
            raise SelectError("invalid socket: not a BaseSocket instance")
        
        if not hasattr(socket, "fileno"):
            raise SelectError("invalid socket: no fileno")

        if socket.closed:
            raise SelectError("invalid socket: socket is closed")
            
    def register(self, socket, mode):
        self._validate(socket)
        if socket in _modes[mode]:
            raise SelectError("socket is already registered")
        
        _modes[mode].append(socket)

    def unregister(self, socket, mode):
        self._validate(socket)
        if socket not in _modes[mode]:
            raise SelectError("socket not registered")
        
        _modes[mode].remove(socket)

    def checkSockets(self):
        for mode in _modes:
            for socket in mode:
                try:
                    _select([socket], [socket], [socket], 0)
                except: # bad socket found
                    mode.remove(socket)
                    socket.close()

    def clean(self):
        for mode in _modes:
            del mode[:]

    def doSelect(self,
                 # A trick to cache globals in local namespace
                 reads=_reads,
                 writes=_writes,
                 select=_select,
                 timeout=0):

        while 1:
            try:
                # Check for write only socks with non-empty buffer
                writes = [s for s in writes if s._buffer]
                r, w, duh = select(reads, writes, [], timeout)
                break
            except ValueError:
                # Something invalid has passed... how ?
                self.checkSockets()
            except TypeError:
                # not a socket, or a closed socket
                self.checkSockets()
            except socket.error, reason:
                code, msg = reason.args
                if code == EINTR:
                    # Interrupted call... 
                    return
                if code == EBADF:
                    # Bad socket... check all and prune it...
                    self.checkSockets()
                else:
                    # I really don't know what's going on...
                    raise
                
        return (r, w)
 

### Loop engine ###

class TaskQueue(object):
    def __init__(self):
        self.tasks = {}

    def __contains__(self, key):
        return key in self.tasks

    def put(self, delay, call, *args, **kwds):
        if not callable(call):
            raise TypeError("call 'must' be a callable object")

        if delay < 0:
            raise ValueError("delay can't be negative")
        
        deadline = time.time() + delay
        self.tasks[deadline] = (call, args, kwds)
        return deadline

    def remove(self, id):
        del self.tasks[id]

    def get(self, id):
        return self.tasks.pop(id)

    def expired(self):
        now = time.time()
        deadlines = self.tasks.keys()
        deadlines.sort()
        for deadline in deadlines:
            if deadline > now:
                raise StopIteration
            yield self.tasks.pop(deadline)

    def __iter__(self):
        return self.expired()

        
class Engine(object):

    timeout = 0.01

    def __init__(self):
        self.queue = TaskQueue()
        self.running = False

    def stop(self):
        self.running = False

    def run(self):
        self.running = True
        while self.running:
            self.update()

    def register(self, delay, call, *args, **kwds):
        return self.queue.put(delay, call, *args, **kwds)

    def unregister(self, id):
        self.queue.remove(id)

    def update(self):
        for call, args, kwds in self.queue:
            try:
                call(*args, **kwds)
            except:
                exc = sys.exc_info()
                self.handleException(*exc)

        time.sleep(self.timeout)

    def handleException(self, exc, value=None, tb=None):
        raise exc, value, tb

    
### Server ###


class TCPServer(object):
    def __init__(self, host=None, type=None, engine=None, select=None):
        self.host = host or ""#socket.getfqdn()
        self.type = type or ServerSocket
        self.engine = engine or Engine()
        self.select = select or Select()
        self.ports = {}

    def clean(self):
        for socket in self.ports.values():
            socket.close()
        self.ports.clear()
        self.select.clean()
        
    def bind(self, port, protocol):
        if not issubclass(protocol, BaseProtocol):
            raise ServerError("protocol is not valid")

        if port in self.ports:
            name = self.ports[port].protocol.__name__
            raise ServerError("port %d already bound with %s"%(port, name))
        
        self.ports[port] = socket = ServerSocket(self, protocol)
        socket.bind(self.host, port)
        socket.listen()
        self.select.register(socket, READ)

    def _accept(self, sock):
        try:
            client = sock.accept()
            self.select.register(client, READ)
            self.select.register(client, WRITE)
        except SelectError:
            pass

    def _read(self, client):
        try:
            client.recv()
        except ConnectionError, reason:
            self._close(client, reason)
        
    def _write(self, client):
        try:
            client.flush()
        except ConnectionError, reason:
            self._close(client, reason)

    def _close(self, client, reason=None):
        # This will assert that any data on the client is flushed
        # before closing
        if client._buffer:
            self.engine.register(0.01, self._close, client, reason=reason)
            return
        # I believe there's no risk of recursive calling because an
        # unhealthy socket (not flushing) will be pruned and closed
        # anyway... but I may be wrong
        try:
            try:
                self.select.unregister(client, READ)
                self.select.unregister(client, WRITE)
            except SelectError:
                pass
        finally:
            client._close(reason)

    def _update(self):
        r, w = self.select.doSelect()

        for sock in r:
            if isinstance(sock, ServerSocket):
                self._accept(sock)
            else:
                self._read(sock)

        for sock in w:
            if sock._buffer:
                self._write(sock)

        self._id = self.engine.register(0, self._update)

    def stop(self):
        self.engine.unregister(self._id)
        self.engine.stop()

    def run(self):
        self._id = self.engine.register(0, self._update)
        self.engine.run()
        

###
