
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

### Imports

import socket
import unittest
import time

import asiel


### Test Exceptions ###

class TestErrorCase(unittest.TestCase):

    def test_MapSocketError(self):
        """Errno codes are mapped to real exceptions"""
        map = asiel._connErrorMap.items()
        for code, exc in map:
            self.assertRaises(exc, self._raise, (code, ""))

    def _raise(self, reason):
        raise asiel.MapSocketError(reason)


### Test Sockets ###

class FakeServer(object):
    host = ""
    def _close(self, client, reason=None):
        client._close(reason)


class ClientTester(object):
    def __init__(self, port=None):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect((HOST, port or PORT))
        self.close = self.socket.close

    def trySend(self):
        sent = self.socket.send("trying to send data")
        return sent

    def tryRecv(self):
        data = self.socket.recv(8192)
        return data


class DataReceived(Exception):
    pass


class ConnectionLost(Exception):
    pass


class DumbProtocol(asiel.RawProtocol):
    def dataReceived(self, data):
        raise DataReceived(data)

    def connectionEstablished(self):
        self._established = True

    def connectionLost(self):
        raise ConnectionLost

# some constants

HOST = socket.gethostname()
PORT = 65004
SERVER = FakeServer()


class TestBaseSocket(unittest.TestCase):
    def setUp(self):
        self.socket = asiel.BaseSocket(SERVER)

    def tearDown(self):
        self.socket.close()

    def testAttributes(self):
        """Attributes are sane"""
        self.assert_(hasattr(self.socket, "fileno"))
        self.assert_(hasattr(self.socket, "socket"))
        self.assert_(hasattr(self.socket, "closed"))
        self.assert_(hasattr(self.socket, "family"))
        self.assert_(hasattr(self.socket, "type"))
        self.assert_(hasattr(self.socket, "attrs"))
        self.assertEqual(self.socket.fileno, self.socket.socket.fileno)

    def testCloseSocket(self):
        """Socket closes"""
        self.socket.close()
        self.assert_(self.socket.closed)

    def testBool(self):
        """Truth testing on socket state"""
        self.assert_(self.socket)
        self.socket.close()
        self.assert_(not self.socket)

    def testServerWeakRef(self):
        """Weak-reference to server managed by 'property'"""
        self.assert_(self.socket.server is SERVER)


class TestServerSocket(TestBaseSocket):
    def setUp(self):
        self.socket = asiel.ServerSocket(SERVER, DumbProtocol)

    def tearDown(self):
        self.socket.close()

    def testAttributes(self):
        """ServerSocket attributes are sane"""
        self.assert_(hasattr(self.socket, "fileno"))
        self.assert_(hasattr(self.socket, "socket"))
        self.assert_(hasattr(self.socket, "closed"))
        self.assert_(hasattr(self.socket, "family"))
        self.assert_(hasattr(self.socket, "type"))
        self.assert_(hasattr(self.socket, "addr"))
        self.assert_(hasattr(self.socket, "attrs"))
        self.assert_(hasattr(self.socket, "name"))
        self.assert_(hasattr(self.socket, "protocol"))
        self.assertEqual(self.socket.fileno, self.socket.socket.fileno)

    def testBindSocket(self):
        """Socket binds to given host and port"""
        self.socket.bind(HOST, PORT)
        self.assertRaises(asiel.SocketError, self.socket.bind, HOST, PORT)
        
    def testListenSocket(self):
        """Socket starts listening"""
        self.socket.bind(HOST, PORT)
        self.socket.listen()
        client = ClientTester()
        client.close()

    def testAcceptSocket(self):
        """Client is accepted and wrapped"""
        self.socket.bind(HOST, PORT)
        self.socket.listen()
        client = ClientTester()
        server = self.socket.accept()
        
        self.assert_(isinstance(server, asiel.ClientSocket))
        client.close()
        self.assertRaises(ConnectionLost, server._close)


class TestClientSocket(unittest.TestCase):
    def setUp(self):
        self.server = asiel.ServerSocket(SERVER, DumbProtocol)
        self.server.bind(HOST, PORT)
        self.server.listen()
        self.remote = ClientTester()
        self.local = self.server.accept()
        self.socket = self.local

    def tearDown(self):
        self.remote.close()
        self.assertRaises(ConnectionLost, self.local.close)
        self.server.close()

    def testAttributes(self):
        """ClientSocket attributes"""
        self.assert_(hasattr(self.local, "fileno"))
        self.assert_(hasattr(self.local, "socket"))
        self.assert_(hasattr(self.local, "closed"))
        self.assert_(hasattr(self.local, "family"))
        self.assert_(hasattr(self.local, "type"))
        self.assert_(hasattr(self.local, "addr"))
        self.assert_(hasattr(self.local, "attrs"))
        self.assert_(hasattr(self.local, "_buffer"))
        self.assert_(self.local._established)
        self.assertEqual(self.local.attrs['host'], self.local.addr[0])
        self.assertEqual(self.local.attrs['port'], self.local.addr[1])

    def testRecvData(self):
        """Data is received"""
        self.remote.trySend()
        self.assertRaises(DataReceived, self.local.recv)

    def testRemoteClosed(self):
        """Local closes when remote drops"""
        self.remote.close()
        self.assertRaises(asiel.SocketError, self.local.recv)

    def testSend(self):
        """Data is sent"""
        data = "Once you start down the dark path..."
        rest = "...forever will it dominate your destiny"
        self.local.send(data)
        self.assertEqual(self.local._buffer, data)
        self.local.send(rest)
        self.assertEqual(self.local._buffer, data+rest)

    def testFlush(self):
        """Data is flushed"""
        data = "Once you start down the dark path, "\
               "forever will it dominate your destiny"
        self.local.send(data)
        self.assertEqual(self.local._buffer, data)
        sent = self.local.flush()
        self.assertEqual(self.local._buffer, data[len(sent):])
        recv = self.remote.tryRecv()
        self.assertEqual(recv, sent)


### Test Protocol ###

class MyBaseProtocol(asiel.BaseProtocol):
    def connectionEstablished(self):
        self.active = True

    def connectionLost(self):
        self.active = False


class TestBaseProtocol(unittest.TestCase):
    def setUp(self):
        self.protocol = MyBaseProtocol()
        self.protocol.connectionEstablished()

    def testActive(self):
        """Protocol instance is initialized"""
        self.assert_(self.protocol.active)

    def testInactive(self):
        """Protocol instance is shutdown"""
        self.protocol.connectionLost()
        self.assert_(not self.protocol.active)


class MyRawProtocol(asiel.RawProtocol):
    def connectionEstablished(self):
        self.active = True

    def connectionLost(self):
        self.active = False

    def dataReceived(self, data):
        self.data = data


class TestRawProtocol(TestBaseProtocol):
    def setUp(self):
        self.protocol = MyRawProtocol()
        self.protocol.connectionEstablished()

    def testDataReceived(self):
        """Protocol receives data"""
        self.protocol.dataReceived("I am a great one duh")
        self.assertEqual(self.protocol.data, "I am a great one duh")

class LineLengthExceeded(ValueError):
    pass

class MyLineProtocol(asiel.LineProtocol):
    line = ""
    def connectionEstablished(self):
        self.active = True

    def connectionLost(self):
        self.active = False
    
    def rawDataReceived(self, data):
        self.data = data

    def lineReceived(self, line):
        self.line = line

    def send(self, line):
        self.sent = line

    def lineLengthExceeded(self, line):
        raise LineLengthExceeded(line)
    

class TestLineProtocol(TestBaseProtocol):
    def setUp(self):
        self.protocol = MyLineProtocol()
        self.protocol.connectionEstablished()

    def testDataReceived(self):
        """Protocol receives data in line mode"""
        self.protocol.dataReceived("I am a great one duh")
        self.assertEqual(self.protocol.line, "")
        self.protocol.dataReceived("\r\n and this will not be processed !")
        self.assertEqual(self.protocol.line, "I am a great one duh")

    def testRawMode(self):
        """Protocol receives data in raw mode"""
        data = "Index\r\nLibrorum\r\nProibitorum\r\n"
        self.protocol.setRawMode()
        self.assert_(not self.protocol.lineMode)
        self.protocol.dataReceived(data)
        self.assertEqual(self.protocol.data, data)

    def testLineOverflow(self):
        """Crashes when line length overflows"""
        limit = self.protocol.MAX
        line = "and what" + ("?" * limit)
        self.assertRaises(LineLengthExceeded, self.protocol.dataReceived, line)
        

### Test Select ###

class FakeServer(object):
    host = socket.gethostname()
    def _close(self, client, reason=None):
        client._close(reason)

HOST = socket.gethostname()
PORT = 65004
SERVER = FakeServer()

class MyProtocol(asiel.RawProtocol):
    def connectionEstablished(self):
        pass

    def dataReceived(self, data):
        pass

    def connectionLost(self):
        pass

class FakeSocket(object):
    def __init__(self, fileno=None):
        self.fileno = fileno or 65536

    def recv(self, bufsize):
        return ""

    def send(self, data):
        return 0

    def close(self):
        pass


class ClientTester(object):
    def __init__(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect((HOST, PORT))
        self.close = self.socket.close

    def trySend(self):
        sent = self.socket.send("trying to send data")
        return sent

    def tryRecv(self):
        data = self.socket.recv(8192)
        return data


class TestSelect(unittest.TestCase):

    def setUp(self):
        self.select = asiel.Select()
        self.server = asiel.ServerSocket(SERVER, MyProtocol)
        self.server.bind(HOST, PORT)
        self.server.listen()

    def tearDown(self):
        self.server._close()
        del asiel._reads[:]
        del asiel._writes[:]

    def testRegisterSocket(self):
        self.select.register(self.server, asiel.READ)
        self.select.register(self.server, asiel.WRITE)
        self.assert_(self.server in asiel._reads)
        self.assert_(self.server in asiel._writes)

    def testRegisterInvalidSocket(self):
        socket = FakeSocket()
        self.assertRaises(asiel.SelectError, self.select.register,
                          socket, asiel.READ)
        self.assertRaises(asiel.SelectError, self.select.register,
                          socket, asiel.WRITE)

    def testRegisterSocketWithNoFileno(self):
        server = self.server
        del server.fileno
        self.assertRaises(asiel.SelectError, self.select.register,
                          server, asiel.READ)
        self.assertRaises(asiel.SelectError, self.select.register,
                          server, asiel.WRITE)

    def testRegisterClosedSocket(self):
        server = self.server
        server.closed = True
        self.assertRaises(asiel.SelectError, self.select.register,
                          server, asiel.READ)
        self.assertRaises(asiel.SelectError, self.select.register,
                          server, asiel.WRITE)

    def testReRegisterSocket(self):
        server = self.server
        self.select.register(server, 0)
        self.select.register(server, 1)
        self.assertRaises(asiel.SelectError, self.select.register,
                          server, asiel.READ)
        self.assertRaises(asiel.SelectError, self.select.register,
                          server, asiel.WRITE)

    def testUnregisterSocket(self):
        self.select.register(self.server, asiel.READ)
        self.select.register(self.server, asiel.WRITE)
        self.select.unregister(self.server, asiel.READ)
        self.select.unregister(self.server, asiel.WRITE)
        self.assert_(self.server not in asiel._reads)
        self.assert_(self.server not in asiel._writes)

    def testUnregisterNonExistantSocket(self):
        server = self.server
        self.assertRaises(asiel.SelectError, self.select.unregister,
                          server, asiel.READ)
        self.assertRaises(asiel.SelectError, self.select.unregister,
                          server, asiel.WRITE)

    def testCheckSocketsI(self):
        socket1 = asiel.ServerSocket(SERVER, MyProtocol)
        self.select.register(socket1, asiel.READ)
        self.select.register(socket1, asiel.WRITE)
        socket2 = asiel.ServerSocket(SERVER, MyProtocol)
        self.select.register(socket2, asiel.READ)
        self.select.register(socket2, asiel.WRITE)
        del socket1.fileno
        self.select.checkSockets()
        self.assert_(socket1 not in asiel._reads)
        self.assert_(socket1 not in asiel._writes)
        self.assert_(socket2 in asiel._reads)
        self.assert_(socket2 in asiel._writes)

    def testCheckSocketsII(self):
        socket1 = asiel.ServerSocket(SERVER, MyProtocol)
        self.select.register(socket1, asiel.READ)
        self.select.register(socket1, asiel.WRITE)
        socket2 = asiel.ServerSocket(SERVER, MyProtocol)
        self.select.register(socket2, asiel.READ)
        self.select.register(socket2, asiel.WRITE)
        socket1.close()
        self.select.checkSockets()
        self.assert_(socket1 not in asiel._reads)
        self.assert_(socket1 not in asiel._writes)
        self.assert_(socket2 in asiel._reads)
        self.assert_(socket2 in asiel._writes)

    def testClean(self):
        socket1 = asiel.ServerSocket(SERVER, MyProtocol)
        self.select.register(socket1, asiel.READ)
        self.select.register(socket1, asiel.WRITE)
        socket2 = asiel.ServerSocket(SERVER, MyProtocol)
        self.select.register(socket2, asiel.READ)
        self.select.register(socket2, asiel.WRITE)
        self.select.clean()
        self.assert_(not asiel._reads)
        self.assert_(not asiel._writes)

    def testDoSelect(self):
        server = self.server
        self.select.register(server, asiel.READ)
        remote = ClientTester()
        reads, writes = self.select.doSelect()
        self.assert_(server in reads and server not in writes)
        local = server.accept()

        self.select.register(local, asiel.READ)
        self.select.register(local, asiel.WRITE)

        reads, writes = self.select.doSelect()
        self.assert_(server not in reads and server not in writes)
        self.assert_(local not in reads and local not in writes)

        remote.trySend()
        reads, writes = self.select.doSelect()
        self.assert_(server not in reads and server not in writes)
        self.assert_(local in reads and local not in writes)
        data = local.recv()
        
        local._buffer = 'NULL'
        reads, writes = self.select.doSelect()
        self.assert_(server not in reads and server not in writes)
        self.assert_(local not in reads)
        self.assert_(local in writes)
        local._buffer = ''
        
        del local.fileno
        reads, writes = self.select.doSelect()
        self.assert_(local not in asiel._reads)
        self.assert_(local not in asiel._writes)
        self.assert_(not reads and not writes)


### Test Loop Engine ###

class TestTaskQueue(unittest.TestCase):
    def setUp(self):
        self.queue = asiel.TaskQueue()

    def _queueCallback(self, *args, **kwds):
        """Helper method"""
        pass

    def testQueueAttributes(self):
        """Base attributes are sane"""
        self.assert_(hasattr(self.queue, "tasks"))

    def registerTask(self, delay=0):
        """Register a task"""
        return self.queue.put(delay, self._queueCallback, 1, 2, a=10, b=20)

    def testRegister(self):
        """Task registration"""
        self.registerTask()

    def testContains(self):
        """'in' keyword works properly"""
        id = self.registerTask()
        self.assert_(id in self.queue)
        
    def testRegisterInvalid(self):
        """Exception raises on invalid task registration"""
        self.assertRaises(TypeError, self.queue.put, 0, None)
        self.assertRaises(ValueError, self.queue.put, -1, type)

    def testDeadlines(self):
        """Deadlines are in the future"""
        now = time.time()
        deadline = self.registerTask()
        self.assert_(deadline > now)
        
    def testDeadlinesDoNotRepeat(self):
        """Two simultaneous tasks do not share deadlines"""
        id1 = self.registerTask()
        id2 = self.registerTask()
        self.assertNotEqual(id1, id2)

    def testUnregister(self):
        """Task unregistration"""
        id = self.registerTask()
        self.queue.remove(id)

    def testInvalidUnregister(self):
        """Exception is raised on unregistering when id does not exist"""
        self.assertRaises(KeyError, self.queue.remove, 1)

    def testGetTask(self):
        """Task is returned when requested"""
        id = self.registerTask()
        call, args, kwds = self.queue.get(id)
        self.assertEqual(call, self._queueCallback, "call")
        self.assertEqual(args, (1, 2), "args")
        self.assertEqual(kwds, {"a":10, "b":20}, "kwds")

    def testTaskRemoved(self):
        """Task is removed correctly"""
        id = self.registerTask()
        call, args, kwds = self.queue.get(id)
        self.assertEqual(call, self._queueCallback, "call")
        self.assertEqual(args, (1, 2), "args")
        self.assertEqual(kwds, {"a":10, "b":20}, "kwds")
        self.assertRaises(KeyError, self.queue.get, id)

    def testGetInvalidTask(self):
        """Exception is raised when getting invalid task"""
        self.assertRaises(KeyError, self.queue.get, 0)

    def testGetIterUpToNow(self):
        """Iterating over queue returns only expired tasks"""
        id1 = self.registerTask()
        id2 = self.registerTask()
        id3 = self.registerTask()
        id4 = self.registerTask(0.01)
        id5 = self.registerTask(1)
        id6 = self.registerTask(2)
        time.sleep(0.1)
        for task in self.queue:
            self.assertEqual(len(task), 3)
        self.assert_(id1 not in self.queue)
        self.assert_(id2 not in self.queue)
        self.assert_(id3 not in self.queue)
        self.assert_(id4 not in self.queue)
        self.assert_(id5 in self.queue)
        self.assert_(id6 in self.queue)


class TestEngine(unittest.TestCase):
    def setUp(self):
        self.engine = asiel.Engine()

    def testEngineAttributes(self):
        """Base attributes"""
        self.assert_(isinstance(self.engine.queue, asiel.TaskQueue))
        self.assert_(not self.engine.running)

    def _engineCallback(self, *args, **kwds):
        """Helper method"""
        self.assertEqual(args, (3,))
        self.assertEqual(kwds, {"a":10})

    def registerTask(self, delay=0):
        """Register a task"""
        return self.engine.register(delay, self._engineCallback, 3, a=10)

    def testRegister(self):
        """Task registration"""
        self.registerTask()

    def testRegisterInvalidCall(self):
        """Exception raises on invalid task registration"""
        self.assertRaises(TypeError, self.engine.register, 0, None)
        self.assertRaises(ValueError, self.engine.register, -1, type)

    def testDeadlines(self):
        """Deadlines are in the future"""
        now = time.time()
        deadline = self.registerTask()
        self.assert_(deadline > now)
        
    def testDeadlinesDoNotRepeat(self):
        """Two simultaneous tasks do not share deadlines"""
        id1 = self.registerTask()
        id2 = self.registerTask()
        self.assertNotEqual(id1, id2)

    def testUnregister(self):
        """Task unregistration"""
        id = self.registerTask()
        self.engine.unregister(id)

    def testInvalidUnregister(self):
        """Exception is raised when id does not exist"""
        self.assertRaises(KeyError, self.engine.unregister, 1)

    def _raiseCallback(self):
        """Helper method: always raise a weird exception"""
        raise ZeroDivisionError("I just raise a exception, nothing more")

    def _nullCallback(self):
        """Helper method: just mark that was called """
        self.called = True

    def _failCallback(self):
        """Helper method: fail imediately"""
        self.fail("Ops... this callback should never be called")

    def testTaskUpdating(self):
        """Update is working"""
        self.called = False
        self.engine.register(0, self._nullCallback)
        self.engine.register(0.1, self._raiseCallback)
        self.engine.register(1, self._failCallback)
        self.engine.update()
        self.assert_(self.called)
        time.sleep(0.2)
        self.assertRaises(ZeroDivisionError, self.engine.update)

    def testExceptionHandling(self):
        """Exceptions are being handled by the 'handleException' method """
        self.engine.register(0, self._raiseCallback)

        def handleException(exc, value=None, tb=None):
            self.assert_(exc is ZeroDivisionError)
            self.assert_(value is not None)
            self.assert_(tb is not None)

        self.engine.handleException = handleException
        self.engine.update()

    def _countCallback(self, *args, **kwds):
        """Helper method: register how many times it was called"""
        self.count += 1
        c = self.count
        self.assertEqual(args, (c,))
        self.assertEqual(kwds, {"a":c*10})
            
    def testEngineRunningAndStopping(self):
        """Engine is running and stopping"""
        self.count = 0
        self.engine.register(0, self._countCallback, 1, a=10)
        self.engine.register(0.02, self._countCallback, 3, a=30)
        self.engine.register(0.06, self._countCallback, 5, a=50)
        self.engine.register(0.04, self._countCallback, 4, a=40)
        self.engine.register(0.08, self._countCallback, 6, a=60)
        self.engine.register(1, self._countCallback, 7, a=70)
        self.engine.register(0, self._countCallback, 2, a=20)
        self.engine.register(0.5, self.engine.stop)
        self.engine.run()
        self.assertEqual(self.count, 6)
        

### Test Server ###

HOST = socket.gethostname()
PORT = 65001


# Global used for client testing

clientData = {"sock":None, 'active':False, 'data':None}


class MyProtocol(asiel.RawProtocol):
    def connectionEstablished(self):
        clientData['sock'] = self
        clientData['active'] = True

    def connectionLost(self):
        clientData['active'] = False

    def dataReceived(self, data):
        clientData['data'] = data


class ClientTester(object):
    def __init__(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect((HOST, PORT))
        self.close = self.socket.close
        self.str = "Wake up... time to die..."
        
    def trySend(self):
        sent = self.socket.send(self.str)
        return sent

    def tryRecv(self):
        data = self.socket.recv(8192)
        return data


class TestBaseServer(unittest.TestCase):
    def setUp(self):
        self.server = asiel.TCPServer(HOST)
        
    def tearDown(self):
        clientData['active'] = False
        clientData['data'] = None
        clientData['sock'] = None
        self.server.clean()
        
    def testServerAttributes(self):
        self.assertEqual(self.server.host, HOST)
        self.assert_(issubclass(self.server.type, asiel.BaseSocket))
        self.assert_(isinstance(self.server.select, asiel.Select))
        self.assert_(isinstance(self.server.engine, asiel.Engine))

class TestServerBindListen(TestBaseServer):
    def testBindServerPort(self):
        self.server.bind(PORT, MyProtocol)
        self.assert_(MyProtocol in self.server.ports[PORT].protocol.__bases__)
        self.assert_(self.server.ports[PORT] in asiel._reads)

    def testRebindServerPort(self):
        self.server.bind(PORT, MyProtocol)
        self.assertRaises(asiel.ServerError, self.server.bind,
                          PORT, MyProtocol)

    def testBindInvalidProtocol(self):
        self.assertRaises(asiel.ServerError, self.server.bind, PORT, object)

    def testServerClean(self):
        self.server.bind(PORT, MyProtocol)
        self.server.clean()
        self.assert_(not self.server.ports)
        self.assert_(not asiel._reads)
        self.assert_(not asiel._writes)


class TestServerCallbacks(TestBaseServer):
    def setUp(self):
        super(TestServerCallbacks, self).setUp()
        self.server.bind(PORT, MyProtocol)
        self.client = ClientTester()

    def tearDown(self):
        super(TestServerCallbacks, self).tearDown()
        self.client.close()

class TestDirectCall(TestServerCallbacks):
    def setUp(self):
        super(TestDirectCall, self).setUp()
        self.server._accept(self.server.ports[PORT])
        self.assert_(clientData['active'])
        self.sock = clientData['sock']
        
    def testAcceptCallback(self):
        self.assert_(self.sock in asiel._reads)
        self.assert_(self.sock in asiel._writes)

    def testCloseClientCallback(self):
        self.server._close(self.sock)
        self.assert_(not self.sock)
        self.assert_(not clientData['active'])
        self.assert_(self.sock.closed)
        self.assert_(self.sock not in asiel._reads)
        self.assert_(self.sock not in asiel._writes)

    def testReadClientCallback(self):
        self.client.trySend()
        self.server._read(self.sock)
        self.assertEqual(clientData['data'], self.client.str) 

    def testWriteClientCallback(self):
        data = "...this piece of junk ?!?!"
        self.sock.send(data)
        self.server._write(self.sock)
        rdata = self.client.tryRecv()
        self.assertEqual(data, rdata)


class TestUpdateCall(TestServerCallbacks):
    def setUp(self):
        super(TestUpdateCall, self).setUp()
        self.server._update()
        self.sock = clientData['sock']
        
    def testAcceptOnUpdate(self):
        self.assert_(self.sock)
        self.assert_(self.sock in asiel._reads)
        self.assert_(self.sock in asiel._writes)
        self.assert_(clientData['active'])

    def testReadOnUpdate(self):
        self.client.trySend()
        self.server._update()
        self.assertEqual(clientData['data'], self.client.str)

    def testWriteOnUpdate(self):
        data = "...this piece of junk ?!?!"
        self.sock.send(data)
        self.server._update()
        rdata = self.client.tryRecv()
        self.assertEqual(data, rdata)

    def testRemoteClosed(self):
        self.client.close()
        self.server._update()
        self.assert_(not clientData['active'])

    def testDataFlushOnClose(self):
        data = "...this piece of junk ?!?!"
        self.sock.send(data)
        self.server._close(self.sock)
        self.server._update()
        rdata = self.client.tryRecv()
        self.assertEqual(data, rdata)
        
class TestServerRun(TestServerCallbacks):
    def setUp(self):
        super(TestServerRun, self).setUp()
        self.loop = self.server.engine.register

    def runAndStop(self):
        self.server.engine.register(1, self.server.stop)
        self.server.run()
        
    def testServerUpdating(self):
        self.runAndStop()
        self.assert_(clientData['active'])

    def _clientInitialized(self):
        self.assert_(clientData['active'])
        sock = clientData['sock']
        self.assert_(sock in asiel._reads)
        self.assert_(sock in asiel._writes)
    
    def testServerContinueUpdating(self):
        self.loop(0.1, self._clientInitialized)
        self.loop(0.2, self.client.close)
        self.runAndStop()
        self.assert_(not clientData['active'])
        sock = clientData['sock']
        self.assert_(sock not in asiel._reads)
        self.assert_(sock not in asiel._writes)
        

if __name__ == "__main__":
    unittest.main()
